@@ -22,7 +22,7 @@ function(depth=3, viewgrid=FALSE, saveg=FALSE)
 		cat("wrote",f,"\n")
 	}
 	tc <-  geometry::delaunayn(g0)
-	tc.surf <- t( surf.tri(g0,tc) )
+	tc.surf <- t( geometry::surf.tri(g0,tc) )
 	s2tess <- list(pc=g0, tcsurf=tc.surf)
 	#---------
 	if(viewgrid) {
