@@ -2,7 +2,7 @@
 ## fslview-compatible gfa-map and V1 volumes 
 
 sph.odfpeaks <-
-function(fbase=NULL, rg=NULL, swap=FALSE, btoption=1, threshold=0.4, showglyph=FALSE, bview="coronal", savedir=tempdir(), order=4)
+function(fbase=NULL, rg=NULL, swap=FALSE, btoption=2, threshold=0.4, showglyph=FALSE, bview="coronal", savedir=tempdir(), order=4)
 {
   bviews <- c("sagittal", "coronal", "axial")
   kv <- match(bview, bviews)
@@ -10,43 +10,38 @@ function(fbase=NULL, rg=NULL, swap=FALSE, btoption=1, threshold=0.4, showglyph=F
   ##-----------
   ## Read data
   testfilexist(fbase=fbase, btoption=btoption)
-  if(btoption == 1) { ## Option 1: S2-shell (DSI 203-point 3mm)
-    btable <- as.matrix(
-      readtable(fbase=fbase, filename="btable.txt"))
-  }
-  else {
-    if(btoption == 2) { 
-      if(is.null(fbase)) {
-				cat("Data files 'data.bval' and 'data.bvec' unspecified !\n") 
-				stop()
-      } else {
-	      bval <- scantable(fbase=fbase, filename="data.bval")
- 	    	# bvec <- readtable(fbase=fbase, filename="data.bvec")
-        bvec <- scantable(fbase=fbase, filename="data.bvec")
-        bvec <- matrix(bvec, ncol=3)
- 	    	btable <- cbind(bval,bvec)
-        rm(bval, bvec)
-			}
+  if(btoption == 1){ ## Option 1: S2-shell (DSI 203-point 3mm)
+    btable <- as.matrix(readtable(fbase=fbase, filename="btable.txt"))
+  } else {
+    if(btoption == 2) { ## Option 2: 3D-dsi grid 
+      bval <- scantable(fbase=fbase, filename="data.bval")
+      # bvec <- readtable(fbase=fbase, filename="data.bvec")
+      bvec <- scantable(fbase=fbase, filename="data.bvec")
+      bvec <- matrix(bvec, ncol=3)
+      btable <- cbind(bval,bvec)
+      rm(bval, bvec)
     }
     else stop()
   }
-	b0 <- which(btable[,1] == 0)
-	odfvertices <- btable[-b0,2:4]
-	tc <-  geometry::delaunayn(odfvertices)
-	tcsurf <- t( surf.tri(odfvertices,tc))	
+  b0 <- which(btable[,1] == 0)
+  odfvertices <- matrix(btable[-b0,2:4], ncol=3)
+  tc <-  geometry::delaunayn(odfvertices)
+  tcsurf <- t( geometry::surf.tri(odfvertices,tc))  
   ##----------------------------
-	gc()
-  cat("Reading data ...")
-  img.nifti  <- readniidata(fbase=fbase, filename="data.nii.gz")
-  volimg <- img.nifti@.Data  
-  mask.nifti <- readniidata(fbase=fbase, filename="data_brain_mask.nii.gz")
-  volmask <- mask.nifti@.Data  
-  rm(img.nifti, mask.nifti)
   gc()
-  ##----------------------------
-  d <- dim(volmask)
-  volgfa <- array(0, dim=d) ## gfas map
-  V1 <- array(0, dim=c(d, 3)) ## V1 direction
+  cat("Reading data ...\n")
+  ptm <- proc.time()
+  niifile  <- readniidata(fbase=fbase, filename="data.nii.gz")
+  volimg <- nifti.image.read(niifile)
+  niimask <- readniidata(fbase=fbase, filename="data_brain_mask.nii.gz")
+  volmask <- nifti.image.read(niimask)
+  print(proc.time() - ptm)
+  gc()
+  d <- dim(volimg)
+  dm <- dim(volmask)
+  volgfa <- array(0, dim=dm)   ## gfas map
+  V1 <- array(0, dim=c(dm, 3)) ## V1 direction
+  # V2 <- array(0, dim=c(dm, 3)) ## V2 direction
   if(is.null(rg)) {
     switch(kv,
       { nslices <- d[1]}, # sagittal,
@@ -55,7 +50,6 @@ function(fbase=NULL, rg=NULL, swap=FALSE, btoption=1, threshold=0.4, showglyph=F
     first <- 1; last <- nslices
   }
   else { first <- rg[1]; last <- rg[2] }
-  cat("\n")
   ##-----------------------------
 	## SPH process preparation
 	gradient <- t(odfvertices)
@@ -176,11 +170,16 @@ function(fbase=NULL, rg=NULL, swap=FALSE, btoption=1, threshold=0.4, showglyph=F
   }
   cat("\n")
   ##-----------------------------
-  f <- paste(savedir,"/data_gfa",sep="")
-  writeNIfTI(volgfa, filename=f, verbose=TRUE)
-  cat("wrote",f,"\n")
-  f <- paste(savedir,"/data_V1",sep="")
-  writeNIfTI(V1, filename=f, verbose=TRUE)
-  cat("wrote",f,"\n")
+  ##  store nifti
+  f <- "data_gfa"
+  niigfa <- niisetup(savedir=savedir, filename=f, dim=dim(volgfa))
+  niigfa[] <- volgfa[]
+  nifti.image.write(niigfa)
+  cat("wrote",file.path(savedir,f),"\n")
+  f <- "data_V1"
+  niiV1 <- niisetup(savedir=savedir, filename=f, dim=dim(V1))
+  niiV1[] <- V1[]
+  nifti.image.write(niiV1)
+  cat("wrote",file.path(savedir,f),"\n")
 }
 
