@@ -1,76 +1,85 @@
 
 sph.odfvxgrid <-
-function(fbase=NULL, rg=c(1,1), swap=FALSE, btoption=1, threshold=0.4, kdir=4, zfactor=5, snapshot=FALSE, showimage="glyphgfa", bview="coronal", savedir=tempdir(), pngfig="odfglyphs", bg="white", order=4, texture=NULL, ...)
+function(fbase=NULL, rg=c(1,1), swap=FALSE, btoption=2, threshold=0.4, kdir=4, zfactor=5, showimage="glyphgfa", bview="coronal", savedir=tempdir(), bg="white", order=4, texture=NULL, ...)
 {
   showimages <- c("none", "gfa", "glyph", "glyphgfa", "glyphrgbmap", "glyphdata") ## map types
   kshow <- match(showimage, showimages)
   bviews <- c("sagittal", "coronal", "axial")
   kv <- match(bview, bviews)
-	stopifnot(is.na(kv) != TRUE)
+  stopifnot(is.na(kv) != TRUE)
   ##-----------
   ## Read data
   testfilexist(fbase=fbase, btoption=btoption)
   if(btoption == 1) { ## Option 1: S2-shell (DSI 203-point 3mm)
-    btable <- as.matrix(
-      readtable(fbase=fbase, filename="btable.txt"))
+    btable <- as.matrix(readtable(fbase=fbase, filename="btable.txt"))
   }
   else {
-    if(btoption == 2) { 
-      if(is.null(fbase)) {
-				cat("Data files 'data.bval' and 'data.bvec' unspecified !\n") 
-				stop()
-      } else {
-	      bval <- scantable(fbase=fbase, filename="data.bval")
- 	    	# bvec <- readtable(fbase=fbase, filename="data.bvec")
-        bvec <- scantable(fbase=fbase, filename="data.bvec")
-        bvec <- matrix(bvec, ncol=3)
- 	    	btable <- cbind(bval,bvec)
-        rm(bval, bvec)
-			}
+    if(btoption == 2) { ## Option 2: 3D-dsi grid 
+      bval <- scantable(fbase=fbase, filename="data.bval")
+      # bvec <- readtable(fbase=fbase, filename="data.bvec")
+      bvec <- scantable(fbase=fbase, filename="data.bvec")
+      bvec <- matrix(bvec, ncol=3)
+      btable <- cbind(bval,bvec)
+      rm(bval, bvec)
     }
     else stop()
   }
-	b0 <- which(btable[,1] == 0)
-	odfvertices <- btable[-b0,2:4]
-	tc <-  geometry::delaunayn(odfvertices)
-	tcsurf <- t( surf.tri(odfvertices,tc))	
   ##----------------------------
-	gc()
-  cat("Reading data ...")
-  img.nifti  <- readniidata(fbase=fbase, filename="data.nii.gz")
-  volimg <- img.nifti@.Data  
-  mask.nifti <- readniidata(fbase=fbase, filename="data_brain_mask.nii.gz")
-  volmask <- mask.nifti@.Data  
-  rm(img.nifti, mask.nifti)
+  b0 <- which(btable[,1] == 0)
+  odfvertices <- btable[-b0,2:4]
+  tc <-  geometry::delaunayn(odfvertices)
+  tcsurf <- t( geometry::surf.tri(odfvertices,tc))  
+  ##----------------------------
   gc()
-  ##----------------------------
-  d <- dim(volmask)
-  # volgfa <- array(0, dim=dim(volmask)) ## gfas map
-  # V1 <- array(0, dim=c(dim(volmask), 3)) ## V1 direction
+  cat("Reading data ...\n")
+  ptm <- proc.time()
+  niifile  <- readniidata(fbase=fbase, filename="data.nii.gz")
+  volimg <- nifti.image.read(niifile)
+  niimask <- readniidata(fbase=fbase, filename="data_brain_mask.nii.gz")
+  volmask <- nifti.image.read(niimask)
+  print(proc.time() - ptm)
+  d <- dim(volimg)
+  dm <- dim(volmask)
   if(is.null(rg)) {
     switch(kv,
       { nslices <- d[1]}, # sagittal,
       { nslices <- d[2]}, # coronal
-      { nslices <- d[3]})  # axial
+      { nslices <- d[3]}) # axial
     first <- 1; last <- nslices
   }
-  else { first <- rg[1]; last <- rg[2] }
-  cat("\n")
+  else {
+    nslices <- diff(rg)+1
+    rx <- rg[1]:rg[2]
+    switch(kv,
+      { d[1] <- nslices; dm[1] <- nslices
+        volimg <- volimg[rx,,,];
+        volmask <- volmask[rx,,]; }, # sagittal,
+      { d[2] <- nslices; dm[2] <- nslices
+        volimg <- volimg[,rx,,]; 
+        volmask <- volmask[,rx,]; }, # coronal
+      { d[3] <- nslices; dm[3] <- nslices
+        volimg <- volimg[,,rx,];
+        volmask <- volmask[,,rx]; })  # axial
+    first <- 1; last <- nslices
+  }
+  volimg <- array(volimg, dim=d)
+  volmask <- array(volmask, dim=dm)
+  gc()
   ##-----------------------------
-	## SPH process preparation
-	gradient <- t(odfvertices)
-	z <- design.spheven(order,gradient,lambda=0.006)
-	plz <- plzero(order)/2/pi
-	ngrad <- dim(gradient)[2]
-	ngrad0 <- ngrad
-	lord <- rep(seq(0,order,2),2*seq(0,order,2)+1)
+  ## SPH process preparation
+  gradient <- t(odfvertices)
+  z <- design.spheven(order,gradient,lambda=0.006)
+  plz <- plzero(order)/2/pi
+  ngrad <- dim(gradient)[2]
+  ngrad0 <- ngrad
+  lord <- rep(seq(0,order,2),2*seq(0,order,2)+1)
   while(length(lord)>=ngrad0){
-	  order <- order-2
-		lord <- rep(seq(0,order,2),2*seq(0,order,2)+1)
- 		cat("Reduced order of spherical harmonics to",order,"\n")
- 	}
-	cat("Using",length(lord),"spherical harmonics\n")
-	L <- -diag(lord*(lord+1)) 
+    order <- order-2
+    lord <- rep(seq(0,order,2),2*seq(0,order,2)+1)
+     cat("Reduced order of spherical harmonics to",order,"\n")
+   }
+  cat("Using",length(lord),"spherical harmonics\n")
+  L <- -diag(lord*(lord+1)) 
   ##-----------------------------
   ## RGB channels - normalize for rgb color
   dt2 <- dim(odfvertices[tcsurf,])[1]
@@ -79,19 +88,19 @@ function(fbase=NULL, rg=c(1,1), swap=FALSE, btoption=1, threshold=0.4, kdir=4, z
     slicedata <- read.slice(img=volimg, mask=volmask, slice=sl, swap=swap, bview=bview)
     ymaskdata <- premask(slicedata)
     if(ymaskdata$empty) next # empty mask
-		maxslicedata <- max(slicedata$niislicets) ##????
-		S <- ymaskdata$yn[-b0,]
-		S <- S / maxslicedata
-		s0 <- 1
-		si <- apply(S, 2, datatrans, s0)
-		sicoef <- z$matrix%*% si
-		sphcoef <- plz%*%L%*%sicoef
-		coef0 <- sphcoef[1,]
-		sphcoef[1,] <- 1/2/sqrt(pi)
-		sphcoef[-1,] <- sphcoef[-1,]/8/pi
-		## odfs
-		odfs <- t(z$design) %*% sphcoef
-		odfs <- apply(odfs, 2, norm01) 
+    maxslicedata <- max(slicedata$niislicets) ##????
+    S <- ymaskdata$yn[-b0,]
+    S <- S / maxslicedata
+    s0 <- 1
+    si <- apply(S, 2, datatrans, s0)
+    sicoef <- z$matrix%*% si
+    sphcoef <- plz%*%L%*%sicoef
+    coef0 <- sphcoef[1,]
+    sphcoef[1,] <- 1/2/sqrt(pi)
+    sphcoef[-1,] <- sphcoef[-1,]/8/pi
+    ## odfs
+    odfs <- t(z$design) %*% sphcoef
+    odfs <- apply(odfs, 2, norm01) 
     ## gfas
     gfas <- apply(odfs, 2, genfa)
     z2d <- ymaskdata$kin
@@ -196,13 +205,6 @@ function(fbase=NULL, rg=c(1,1), swap=FALSE, btoption=1, threshold=0.4, kdir=4, z
          par3d('windowRect'=c(0,0,600,600), 'zoom'=0.6, skipRedraw=FALSE)
           rgl.bringtotop()
       }
-      if(snapshot) {
-        ## sp <- tempfile(pattern="maplines", tmpdir=savedir, fileext=".png")
-        sp <- paste(savedir,"/",pngfig,".png", sep="")
-        readline("Prepare zoom for taking rgl.snapshot and press return ... ")
-        rgl.snapshot(sp)
-        cat("snapshot file", sp,"\n")
-      }
     }
     # print(proc.time() - ptm)
     ##---
@@ -213,4 +215,6 @@ function(fbase=NULL, rg=c(1,1), swap=FALSE, btoption=1, threshold=0.4, kdir=4, z
     }
   }
   cat("\n")
+  rm(list=ls())
+  gc()
 }
